# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['tf_transformers',
 'tf_transformers.activations',
 'tf_transformers.callbacks',
 'tf_transformers.callbacks.metrics',
 'tf_transformers.core',
 'tf_transformers.data',
 'tf_transformers.data.callbacks',
 'tf_transformers.data.processors',
 'tf_transformers.layers',
 'tf_transformers.layers.attention',
 'tf_transformers.layers.mask',
 'tf_transformers.layers.transformer',
 'tf_transformers.losses',
 'tf_transformers.models',
 'tf_transformers.models.albert',
 'tf_transformers.models.bart',
 'tf_transformers.models.bert',
 'tf_transformers.models.bigbird',
 'tf_transformers.models.clip',
 'tf_transformers.models.encoder_decoder',
 'tf_transformers.models.gpt2',
 'tf_transformers.models.model_configs',
 'tf_transformers.models.model_configs.albert',
 'tf_transformers.models.model_configs.bert',
 'tf_transformers.models.model_configs.gpt2',
 'tf_transformers.models.model_configs.mt5',
 'tf_transformers.models.model_configs.roberta',
 'tf_transformers.models.model_configs.t5',
 'tf_transformers.models.mt5',
 'tf_transformers.models.roberta',
 'tf_transformers.models.t5',
 'tf_transformers.models.tasks',
 'tf_transformers.models.vit',
 'tf_transformers.notebooks.experimental',
 'tf_transformers.notebooks.tutorials.albert_pretrain',
 'tf_transformers.optimization',
 'tf_transformers.pipeline',
 'tf_transformers.text',
 'tf_transformers.text.lm_tasks',
 'tf_transformers.utils']

package_data = \
{'': ['*'],
 'tf_transformers': ['notebooks/*',
                     'notebooks/conversion_scripts/*',
                     'notebooks/tutorials/*',
                     'notebooks/tutorials/joint_loss_experiments/*',
                     'notebooks/tutorials/joint_loss_experiments/glue/*'],
 'tf_transformers.models.model_configs': ['unilm_cnndm/*']}

install_requires = \
['absl-py>=1.0.0,<2.0.0',
 'sentencepiece>=0.1.96,<0.2.0',
 'tqdm>=4.62.3,<5.0.0',
 'transformers>=4.15.0,<5.0.0']

setup_kwargs = {
    'name': 'tf-transformers',
    'version': '1.0.19',
    'description': 'NLP with Transformer based models on Tensorflow 2.0',
    'long_description': '<!---\nCopyright 2021 The LegacyAI Team. All rights reserved.\n\nLicensed under the Apache License, Version 2.0 (the "License");\nyou may not use this file except in compliance with the License.\nYou may obtain a copy of the License at\n\n    http://www.apache.org/licenses/LICENSE-2.0\n\nUnless required by applicable law or agreed to in writing, software\ndistributed under the License is distributed on an "AS IS" BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\nSee the License for the specific language governing permissions and\nlimitations under the License.\n-->\n\n<p align="center">\n    <br>\n    <img src="src/logo2.png" width="400"/>\n    <br>\n</p>\n\n<p align="center">\n<a href="https://github.com/legacyai/tf-transformers/actions?workflow=Tests">\n        <img alt="Tests" src="https://github.com/legacyai/tf-transformers/workflows/Tests/badge.svg">\n</a>\n\n<a href="https://codecov.io/gh/legacyai/tf-transformers">\n        <img alt="Coverage" src="https://codecov.io/gh/legacyai/tf-transformers/branch/main/graph/badge.svg?token=9TZ10G9GL6">\n</a>\n\n<a href="https://opensource.org/licenses/Apache-2.0">\n        <img alt="License" src="https://img.shields.io/badge/License-Apache%202.0-blue.svg">\n</a>\n</p>\n\n<h3 align="center">\n<p>tf-transformers: faster and easier state-of-the-art NLP in TensorFlow 2.0\n</h3>\n\n\ntf-transformers is designed to harness the full power of Tensorflow 2, to make it much faster and simpler comparing to existing Tensorflow based NLP architectures. On an average, there is **80 %** improvement over current exsting Tensorflow based libraries, on text generation and other tasks. You can find more details in the Benchmarks section.\n\nAll / Most NLP downstream tasks can be easily integrated into Tranformer based models with much ease. All the models can be trained using ```model.fit```, which supports **GPU**, **multi-GPU**, **TPU**.\n\n## Version\nversion: v1.0.8\n## Unique Features\n- **Faster AutoReggressive Decoding** using Tensorflow2. Faster than PyTorch in most experiments (V100 GPU). **80%** faster compared to existing TF based libararies (relative difference) Refer [benchmark code](tests/notebooks/benchmarks/).\n- Complete **TFlite** support for **BERT, RoBERTA, T5, Albert, mt5** for all down stream tasks except text-generation\n- **Faster sentence-piece alignment** (no more LCS overhead)\n- **Variable batch text generation** for Encoder only models like GPT2\n- No more hassle of writing long codes for **TFRecords. minimal and simple**.\n- Off the shelf support for auto-batching **tf.data.dataset** or **tf.ragged** tensors\n- Pass dictionary outputs directly to loss functions inside ```tf.keras.Model.fit``` using **model.compile2** . Refer [examples](src/tf_transformers/notebooks/tutorials/) or [blog](https://legacyai-org.medium.com/tf-transformers-f7722536ba61)\n- Multiple mask modes like **causal**, **user-defined**, **prefix** by changing one argument . Refer [examples](src/tf_transformers/notebooks/tutorials/) or [blog](https://legacyai-org.medium.com/tf-transformers-f7722536ba61)\n\n\n## Performance Benchmarks\n\nEvaluating performance benhcmarks is trickier. I evaluated **tf-transformers**, primarily on **text-generation** tasks with **GPT2 small** and **t5 small**, with amazing **HuggingFace**, as it is the ready to go library for NLP right now. Text generation tasks require efficient caching to make use of past **Key** and **Value** pairs.\n\nOn an average, **tf-transformers** is **80 %** faster than **HuggingFace** **Tensorflow** implementation and in most cases it is **comparable** or **faster** than **PyTorch**.\n\n#### 1. GPT2 benchmark\n\nThe evaluation is based on average of 5 runs, with different **batch_size**, **beams**, **sequence_length** etc. So, there is qute a larg combination, when it comes to **BEAM** and **top-k*8 decoding. The figures are **randomly taken 10 samples**. But, you can see the full code and figures in the repo.\n\n* GPT2 greedy\n\n<p align="left">\n    <br>\n    <img src="tests/notebooks/benchmarks/gpt2/gpt2_greedy_sample.png" width="900"/>\n    <br>\n<p>\n\n* GPT2 beam\n\n<p align="left">\n    <br>\n    <img src="tests/notebooks/benchmarks/gpt2/gpt2_beam_sample.png" width="900"/>\n    <br>\n<p>\n\n* GPT2 top-k top-p\n\n<p align="left">\n    <br>\n    <img src="tests/notebooks/benchmarks/gpt2/gpt2_topk_top_p_sample.png" width="900"/>\n    <br>\n<p>\n\n* GPT2 greedy histogram\n\n<p align="left">\n    <br>\n    <img src="tests/notebooks/benchmarks/gpt2/greedy.png" width="500"/>\n    <br>\n<p>\n\n[Codes to reproduce GPT2 benchmark experiments](tests/notebooks/benchmarks/gpt2)\n\n[Codes to reproduce T5 benchmark experiments](tests/notebooks/benchmarks/t5)\n\n## QuickStart\n\nI am providing some basic tutorials here, which covers basics of tf-transformers and how can we use it for other downstream tasks. All/most tutorials has following structure:\n\n* Introduction About the Problem\n* Prepare Training Data\n* Load Model and asociated downstream Tasks\n* Define Optimizer, Loss\n* Train using Keras and CustomTrainer\n* Evaluate Using Dev data\n* In Producton - Secton defines how can we use ```tf.saved_model``` in production + pipelines\n\n## Production Ready Tutorials\n\nStart by converting **HuggingFace** models (base models only) to **tf-transformers** models.\n\nHere are a few examples : Jupyter Notebooks:\n\n- [Basics of tf-transformers](src/tf_transformers/notebooks/tutorials/basics.ipynb)\n- [Convert HuggingFace Models ( BERT, Albert, Roberta, GPT2, t5, mt5) to tf-transformers checkpoints](src/tf_transformers/notebooks/conversion_scripts/)\n- [Name Entity Recognition + Albert + TFlite + Joint Loss + Pipeline](src/tf_transformers/notebooks/tutorials/ner_albert.ipynb)\n- [Squad v1.1 + Roberta + TFlite + Pipeline](src/tf_transformers/notebooks/tutorials/squad_roberta.ipynb)\n- [Roberta2Roberta Encoder Decoder + XSUM + Summarisation](src/tf_transformers/notebooks/tutorials/seq2seq_summarization.ipynb)\n- [Squad v1.1 + T5 + Text Generation](src/tf_transformers/notebooks/tutorials/t5_squad_as_generation.ipynb)\n- [Squad v1.1 + T5 + Span Selection + TFlite + Pipeline](src/tf_transformers/notebooks/tutorials/t5_squad_span_selection.ipynb)\n- [Albert + GLUE + Joint Loss - Glue Score 81.0 on 14 M parameter + 5 layers](src/tf_transformers/notebooks/tutorials/joint_loss_experiments)\n- [Albert + Squad + Joint Loss - EM/F1 78.1/87.0 on 14 M parameter + 5 layers](src/tf_transformers/notebooks/tutorials/joint_loss_experiments/squad.ipynb)\n- [Squad v1.1 + GPT2 + Causal Masking EM/F1 37.36/50.20] (Coming Soon)\n- [Squad v1.1 + GPT2 + Prefix Masking EM/F1 47.52/63.20](Coming Soon)\n- BERT + STS-B + Regression (Coming Soon)\n- [BERT + COLA  + Text Classification + TFlite + Pipeline](src/tf_transformers/notebooks/tutorials/)\n\n## Why should I use tf-transformers?\n\n1. Use state-of-the-art models in Production, with less than 10 lines of code.\n    - High performance models, better than all official Tensorflow based models\n    - Very simple classes for all downstream tasks\n    - Complete TFlite support for all tasks except text-generation\n\n2. Make industry based experience to avaliable to students and community with clear tutorials\n\n3. Train any model on **GPU**, **multi-GPU**, **TPU** with amazing ```tf.keras.Model.fit```\n    - Train state-of-the-art models in few lines of code.\n    - All models are completely serializable.\n\n\n4. Customize any models or pipelines with minimal or no code change.\n\n## Do we really need to distill? Jont Loss is all we need.\n\n#### 1. GLUE\nWe have conducted few experiments to squeeze the power of **Albert base** models ( concept is applicable to any models and in tf-transformers, it is out of the box.)\n\nThe idea is minimize the loss for specified task in each layer of your model and check predictions at each layer. as per our experiments, we are able to get the best smaller model (thanks to **Albert**), and from **layer 4** onwards we beat all the smaller model in **GLUE** benchmark. By **layer 6**, we got a **GLUE** score of **81.0**, which is **4** points ahead of **Distillbert** with GLUE score of **77** and **MobileBert** GLUE score of **78**.\n\nThe **Albert** model has **14 million** parameters, and by using **layer 6**, we were able to speed up the compuation by 50% .\n\nThe concept is applicable to all the models.\n\n[Codes to reproduce GLUE Joint Loss experiments](src/tf_transformers/notebooks/tutorials/joint_loss_experiments/glue/)\n\n\n<p align="left">\n    <br>\n    Benchmark Results\n    <img src="src/tf_transformers/notebooks/tutorials/joint_loss_experiments/glue/glue_benchmark.png" width="700"/>\n    <br>\n<p>\n\n* GLUE score ( not including WNLI )\n\n#### 2. SQUAD v1.1\n\nWe have trained Squad v1.1 with joint loss. At **layer 6** we were able to achieve same performance as of **Distillbert** - (**EM - 78.1 and F1 - 86.2**), but slightly worser than **MobileBert**.\n\nBenchmark Results\n<p align="left">\n    <br>\n    <img src="src/tf_transformers/notebooks/tutorials/joint_loss_experiments/squad_benchmark.png" width="200"/>\n    <br>\n<p>\n\n[Codes to reproduce Squad v1.1 Joint Loss experiments](src/tf_transformers/notebooks/tutorials/joint_loss_experiments/squad.ipynb\n)\n\n**Note**: We have a new model in pipeline. :-)\n## Installation\n\n### With pip\n\nThis repository is tested on Python 3.7+, and Tensorflow 2.3.1\n\nRecommended to use a [virtual environment](https://docs.python.org/3/library/venv.html).\n\nAssuming Tensorflow 2.0 is installed\n\n```bash\npip install tf-transformers\n```\n\n### From Github\n\nAssuming poetry is installed. If not ```pip install poetry``` .\n\n```git clone https://github.com/legacyai/tf-transformers.git```\n\n```cd tf-transformers```\n\n```poetry install```\n\n### Pipeline\n\nPipeline in tf-transformers is different from HuggingFace. Here, pipeline for specific tasks expects a **model** and **tokenizer_fn**. Because in an ideal scenario, no one will be able to understand whats the kind of **pre-processing** we want to do to our inputs. Please refer above tutorial notebooks for examples.\n\n**Token Classificaton Pipeline (NER)**\n``` python\n\nfrom tf_transformers.pipeline import Token_Classification_Pipeline\n\ndef tokenizer_fn(feature):\n    """\n    feature: tokenized text (tokenizer.tokenize)\n    """\n    result = {}\n    result["input_ids"] = tokenizer.convert_tokens_to_ids([tokenizer.cls_token] +  feature[\'input_ids\'] + [tokenizer.bos_token])\n    result["input_mask"] = [1] * len(result["input_ids"])\n    result["input_type_ids"] = [0] * len(result["input_ids"])\n    return result\n\n# load Keras/ Serialized Model\nmodel_ner = # Load Model\nslot_map_reverse = # dictionary index - entity mapping\npipeline = Token_Classification_Pipeline( model = model_ner,\n                tokenizer = tokenizer,\n                tokenizer_fn = tokenizer_fn,\n                SPECIAL_PIECE = SPIECE_UNDERLINE,\n                label_map = slot_map_reverse,\n                max_seq_length = 128,\n                batch_size=32)\n\nsentences = [\'I would love to listen to Carnatic music by Yesudas\',\n            \'Play Carnatic Fusion by Various Artists\',\n            \'Please book 2 tickets from Bangalore to Kerala\']\nresult = pipeline(sentences)\n```\n\n**Span Selection Pipeline (QA)**\n``` python\n\nfrom tf_transformers.pipeline import Span_Extraction_Pipeline\n\ndef tokenizer_fn(features):\n    """\n    features: dict of tokenized text\n    Convert them into ids\n    """\n\n    result = {}\n    input_ids = tokenizer.convert_tokens_to_ids(features[\'input_ids\'])\n    input_type_ids = tf.zeros_like(input_ids).numpy().tolist()\n    input_mask = tf.ones_like(input_ids).numpy().tolist()\n    result[\'input_ids\'] = input_ids\n    result[\'input_type_ids\'] = input_type_ids\n    result[\'input_mask\'] = input_mask\n    return result\n\nmodel = # Load keras/ saved_model\n# Span Extraction Pipeline\npipeline = Span_Extraction_Pipeline(model = model,\n                tokenizer = tokenizer,\n                tokenizer_fn = tokenizer_fn,\n                SPECIAL_PIECE = ROBERTA_SPECIAL_PEICE,\n                n_best_size = 20,\n                n_best = 5,\n                max_answer_length = 30,\n                max_seq_length = 384,\n                max_query_length=64,\n                doc_stride=20)\n\n\nquestions = [\'When was Kerala formed?\']\ncontexts = [\'\'\'Kerala (English: /ˈkɛrələ/; Malayalam: [ke:ɾɐɭɐm] About this soundlisten (help·info)) is a state on the southwestern Malabar Coast of India. It was formed on 1 November 1956, following the passage of the States Reorganisation Act, by combining Malayalam-speaking regions of the erstwhile states of Travancore-Cochin and Madras. Spread over 38,863 km2 (15,005 sq mi), Kerala is the twenty-first largest Indian state by area. It is bordered by Karnataka to the north and northeast, Tamil Nadu to the east and south, and the Lakshadweep Sea[14] to the west. With 33,387,677 inhabitants as per the 2011 Census, Kerala is the thirteenth-largest Indian state by population. It is divided into 14 districts with the capital being Thiruvananthapuram. Malayalam is the most widely spoken language and is also the official language of the state.[15]\'\'\']\nresult = pipeline(questions=questions, contexts=contexts)\n\n```\n\n**Classification Model Pipeline**\n``` python\nfrom tf_transformers.pipeline import Classification_Pipeline\nfrom tf_transformers.data import pad_dataset_normal\n\ntokenizer = BertTokenizer.from_pretrained("bert-base-uncased")\nmax_seq_length = 128\n\n@pad_dataset_normal\ndef tokenizer_fn(texts):\n    """\n    feature: tokenized text (tokenizer.tokenize)\n    pad_dataset_noral will automatically pad it.\n    """\n    input_ids = []\n    input_type_ids = []\n    input_mask = []\n    for text in texts:\n        input_ids_ex = [tokenizer.cls_token] + tokenizer.tokenize(text)[: max_seq_length-2] + [tokenizer.sep_token] # -2 to add CLS and SEP\n        input_ids_ex = tokenizer.convert_tokens_to_ids(input_ids_ex)\n        input_mask_ex = [1] * len(input_ids_ex)\n        input_type_ids_ex = [0] * len(input_ids_ex)\n\n        input_ids.append(input_ids_ex)\n        input_type_ids.append(input_type_ids_ex)\n        input_mask.append(input_mask_ex)\n\n    result = {}\n    result[\'input_ids\'] = input_ids\n    result[\'input_type_ids\'] = input_type_ids\n    result[\'input_mask\'] = input_mask\n    return result\n\nmodel = # Load keras/ saved_model\nlabel_map_reverse = {0: \'unacceptable\', 1: \'acceptable\'}\npipeline = Classification_Pipeline( model = model,\n                tokenizer_fn = tokenizer_fn,\n                label_map = label_map_reverse,\n                batch_size=32)\n\nsentences = [\'In which way is Sandy very anxious to see if the students will be able to solve the homework problem?\',\n            \'The book was written by John.\',\n            \'Play Carnatic Fusion by Various Artists\',\n            \'She voted herself.\']\nresult = pipeline(sentences)\n```\n## Supported Models architectures\n\ntf-transformers currently provides the following architectures .\n1. **[ALBERT](https://huggingface.co/transformers/model_doc/albert.html)** (from Google Research and the Toyota Technological Institute at Chicago) released with the paper [ALBERT: A Lite BERT for Self-supervised Learning of Language Representations](https://arxiv.org/abs/1909.11942), by Zhenzhong Lan, Mingda Chen, Sebastian Goodman, Kevin Gimpel, Piyush Sharma, Radu Soricut.\n2. **[BERT](https://huggingface.co/transformers/model_doc/bert.html)** (from Google) released with the paper [BERT: Pre-training of Deep Bidirectional Transformers for Language Understanding](https://arxiv.org/abs/1810.04805) by Jacob Devlin, Ming-Wei Chang, Kenton Lee and Kristina Toutanova.\n3. **[BERT For Sequence Generation](https://huggingface.co/transformers/model_doc/bertgeneration.html)** (from Google) released with the paper [Leveraging Pre-trained Checkpoints for Sequence Generation Tasks](https://arxiv.org/abs/1907.12461) by Sascha Rothe, Shashi Narayan, Aliaksei Severyn.\n4. **[ELECTRA](https://huggingface.co/transformers/model_doc/electra.html)** (from Google Research/Stanford University) released with the paper [ELECTRA: Pre-training text encoders as discriminators rather than generators](https://arxiv.org/abs/2003.10555) by Kevin Clark, Minh-Thang Luong, Quoc V. Le, Christopher D. Manning.\n5. **[GPT-2](https://huggingface.co/transformers/model_doc/gpt2.html)** (from OpenAI) released with the paper [Language Models are Unsupervised Multitask Learners](https://blog.openai.com/better-language-models/) by Alec Radford*, Jeffrey Wu*, Rewon Child, David Luan, Dario Amodei** and Ilya Sutskever**.\n6. **[MT5](https://huggingface.co/transformers/model_doc/mt5.html)** (from Google AI) released with the paper [mT5: A massively multilingual pre-trained text-to-text transformer](https://arxiv.org/abs/2010.11934) by Linting Xue, Noah Constant, Adam Roberts, Mihir Kale, Rami Al-Rfou, Aditya Siddhant, Aditya Barua, Colin Raffel.\n7. **[RoBERTa](https://huggingface.co/transformers/model_doc/roberta.html)** (from Facebook), released together with the paper a [Robustly Optimized BERT Pretraining Approach](https://arxiv.org/abs/1907.11692) by Yinhan Liu, Myle Ott, Naman Goyal, Jingfei Du, Mandar Joshi, Danqi Chen, Omer Levy, Mike Lewis, Luke Zettlemoyer, Veselin Stoyanov.\n8. **[T5](https://huggingface.co/transformers/model_doc/t5.html)** (from Google AI) released with the paper [Exploring the Limits of Transfer Learning with a Unified Text-to-Text Transformer](https://arxiv.org/abs/1910.10683) by Colin Raffel and Noam Shazeer and Adam Roberts and Katherine Lee and Sharan Narang and Michael Matena and Yanqi Zhou and Wei Li and Peter J. Liu.\n\n## Note\n\n```tf-transformers``` is a personal project. This has nothing to do with any organization. So, I might not be able to host equivalent ```checkpoints``` of all base models. As a result, there is a [**conversion**](src/tf_transformers/notebooks/conversion_scripts) notebooks, to convert above mentioned architectures from **HuggingFace** to **tf-transformers**.\n\n## Credits\n\nI want to give credits to [Tensorflow NLP official repository](https://github.com/tensorflow/models/tree/master/official/nlp). I used November 2019 version of master branch ( where ```tf.keras.Network```) was used for models. I have modified that by large extend now.\n\nApart from that, I have used many common scripts from many open repos. I might not be able to recall everything as it is. But still credit goes to them  too.\n\n## Citation\n\n:-)\n',
    'author': 'Sarath R Nair',
    'author_email': 's4sarath@gmail.com',
    'maintainer': 'Sarath R Nair',
    'maintainer_email': 's4sarath@gmail.com',
    'url': '',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
